function [dataSet, labeledDataSet, hmmDataSet, setOfStates, setOfVs, setOfVsInMap] = weatherData(filepath, q)

import java.util.HashMap;

% reading text file
[C1 C2 C3 C4 C5 C6 C7 C8 C9 C10 C11 C12] = textread(filepath, '%s%s%s%s%s%s%s%s%s%s%s%s', 'delimiter', ',');

% selection of columns for the data set
dataSet = [C2 C4 C5 C8 C12]; % temperature, humidity, preasure, wind speed, condition
[I, n] = size(dataSet);
n = n - 1;
conditionColumn = 5;

% copying data set to a numeric matrix
tempDataSet = zeros(I, n);
for i = 1 : I   
    for j = 1 : n
        if strcmp(dataSet{i, j}, 'Calm')
            tempDataSet(i, j) = 0;
        else
            tempDataSet(i, j) = str2num(dataSet{i, j});
        end
    end
    
    
    % grouping conditions
    if strcmp(dataSet{i, conditionColumn}, 'Fog') | strcmp(dataSet{i, conditionColumn}, 'Mist') ...
        | strcmp(dataSet{i, conditionColumn}, 'Shallow Fog') | strcmp(dataSet{i, conditionColumn}, 'Heavy Fog') ...
        | strcmp(dataSet{i, conditionColumn}, 'Patches of Fog') | strcmp(dataSet{i, conditionColumn}, 'Light Freezing Fog')
        dataSet{i, conditionColumn} = 'Fog';
    elseif strcmp(dataSet{i, conditionColumn}, 'Light Rain') | strcmp(dataSet{i, conditionColumn}, 'Light Rain Showers') ...
            | strcmp(dataSet{i, conditionColumn}, 'Light Thunderstorms and Rain') ...
            | strcmp(dataSet{i, conditionColumn}, 'Drizzle') | strcmp(dataSet{i, conditionColumn}, 'Light Drizzle')
        dataSet{i, conditionColumn} = 'Light Rain';
    elseif strcmp(dataSet{i, conditionColumn}, 'Rain') | strcmp(dataSet{i, conditionColumn}, 'Rain Showers') ...
            | strcmp(dataSet{i, conditionColumn}, 'Thunderstorms and Rain') | strcmp(dataSet{i, conditionColumn}, 'Heavy Rain Showers')
        dataSet{i, conditionColumn} = 'Rain';
    elseif strcmp(dataSet{i, conditionColumn}, 'Mostly Cloudy') | strcmp(dataSet{i, conditionColumn}, 'Partly Cloudy') | strcmp(dataSet{i, conditionColumn}, 'Scattered Clouds')
        dataSet{i, conditionColumn} = 'Cloudy';
    end
end

% finding mins and maxes of columns
mins = zeros(1, n);
maxes = zeros(1, n);
for i = 1 : n    
    mins(1, i) = min(tempDataSet(:, i));
    maxes(1, i) = max(tempDataSet(:, i));
end

% discretizing data set
setOfStates = unique(dataSet(:, conditionColumn));
labeledDataSet = zeros(I, n + 1);
for i = 1 : I
    for j = 1 : n
        labeledDataSet(i, j) = ceil(q * (tempDataSet(i, j) - mins(j)) / (maxes(j) - mins(j)));        
    end
    labeledDataSet(i, n + 1) = find(strcmp(setOfStates, dataSet{i, n + 1}));
end

setOfVs = {'*'};
setOfVsInMap = HashMap;
setOfVsInMap.put('*', 1); % fake observation for all unknowns

hmmDataSet = zeros(I, 2); % two columns, observation index in the first one, state index in the second one.
for i = 1 : I
    observation = mat2str(labeledDataSet(i, 1 : n));
    if ~setOfVsInMap.containsKey(observation)            
        setOfVsInMap.put(observation, setOfVsInMap.size() + 1);        
        setOfVs{length(setOfVs) + 1} = observation;
    end
    observationIndex = setOfVsInMap.get(observation);
    hmmDataSet(i, 1) = observationIndex;
    hmmDataSet(i, 2) = labeledDataSet(i, n + 1);
end
